@@ -7,7 +7,7 @@ from functools import reduce
 
 import attr
 
-from . import biblio, config, h, messages as m, t
+from . import biblio, config, h, messages as m, refs, t
 from .line import Line
 
 
@@ -18,22 +18,25 @@ from .line import Line
 # if you're outputting a raw element (<pre>/etc),
 # just spam it onto the first line.
 
-InfoTreeT: t.TypeAlias = "list[defaultdict[str, list[str]]]"
+if t.TYPE_CHECKING:
+    InfoTreeT: t.TypeAlias = list[defaultdict[str, list[str]]]
+
+    class TransformFuncT(t.Protocol):
+        def __call__(
+            self, lines: list[str], tagName: str, firstLine: str, lineNum: int | None, doc: t.SpecT
+        ) -> list[str]:
+            ...
 
 
-class TransformFuncT(t.Protocol):
-    def __call__(self, lines: list[str], tagName: str, firstLine: str, lineNum: int | None, doc: t.SpecT) -> list[str]:
+if t.TYPE_CHECKING:
+
+    @t.overload
+    def transformDataBlocks(doc: t.SpecT, lines: list[Line]) -> list[Line]:
         ...
 
-
-@t.overload
-def transformDataBlocks(doc: t.SpecT, lines: list[Line]) -> list[Line]:
-    ...
-
-
-@t.overload
-def transformDataBlocks(doc: t.Spec, lines: list[str]) -> list[str]:
-    ...
+    @t.overload
+    def transformDataBlocks(doc: t.Spec, lines: list[str]) -> list[str]:
+        ...
 
 
 def transformDataBlocks(doc: t.SpecT, lines: list[Line] | list[str]) -> list[Line] | list[str]:
@@ -193,7 +196,8 @@ def commonPrefix(line1: str, line2: str) -> str:
 
 
 def getWsPrefix(line: str) -> str:
-    return t.cast("re.Match", re.match(r"(\s*)", line)).group(1)
+    match = t.cast("re.Match", re.match(r"(\s*)", line))
+    return t.cast(str, match.group(1))
 
 
 def transformPre(lines: list[str], tagName: str, firstLine: str, lineNum: int | None, doc: t.SpecT) -> list[str]:
@@ -484,7 +488,7 @@ def transformArgumentdef(
     attrs = parseDefBlock(lines, "argumentdef", capitalizeKeys=False, lineNum=lineNum)
     el = h.parseHTML(firstLine + "</pre>")[0]
     if "for" in el.attrib:
-        forValue = el.get("for")
+        forValue = t.cast(str, el.get("for"))
         el.set("data-dfn-for", forValue)
         if "/" in forValue:
             interface, method = forValue.split("/")
@@ -495,8 +499,8 @@ def transformArgumentdef(
     else:
         m.die("Argumentdef blocks need a for='' attribute specifying their method.", lineNum=lineNum)
         return []
-    h.addClass(el, "data")
-    rootAttrs = " ".join("{}='{}'".format(k, h.escapeAttr(v)) for k, v in el.attrib.items())
+    h.addClass(doc, el, "data")
+    rootAttrs = " ".join(f"{str(k)}='{h.escapeAttr(str(v))}'" for k, v in el.attrib.items())
     text = (
         """
 <table {attrs}{lineNumAttr}>
@@ -664,8 +668,11 @@ def processAnchors(anchors: InfoTreeT, doc: t.SpecT, lineNum: int | None = None)
             urlSuffix = anchor["url"][0]
         else:
             urlSuffix = config.simplifyText(anchor["text"][0])
-        url = urlPrefix + ("" if "#" in urlPrefix or "#" in urlSuffix else "#") + urlSuffix
-        status = "local"
+        if "#" in urlPrefix or "#" in urlSuffix or urlPrefix == "":
+            urlJoiner = ""
+        else:
+            urlJoiner = "#"
+        url = urlPrefix + urlJoiner + urlSuffix
         shortname = None
         level = None
         if "shortname" in anchor and "level" in anchor:
@@ -700,17 +707,20 @@ def processAnchors(anchors: InfoTreeT, doc: t.SpecT, lineNum: int | None = None)
         if anchor["type"][0] in config.lowercaseTypes:
             anchor["text"][0] = anchor["text"][0].lower()
         doc.refs.anchorBlockRefs.refs[anchor["text"][0]].append(
-            {
-                "linkingText": anchor["text"][0],
-                "type": anchor["type"][0].lower(),
-                "url": url,
-                "shortname": shortname.lower() if shortname is not None else doc.md.shortname,
-                "level": level if level is not None else doc.md.level,
-                "for": anchor.get("for", []),
-                "export": True,
-                "status": status,
-                "spec": spec.lower() if spec is not None else "",
-            }
+            refs.RefWrapper(
+                anchor["text"][0],
+                {
+                    "type": anchor["type"][0].lower(),
+                    "url": url,
+                    "shortname": shortname.lower() if shortname is not None else doc.md.shortname,
+                    "level": level if level is not None else doc.md.level,
+                    "for_": anchor.get("for", []),
+                    "export": True,
+                    "normative": True,
+                    "status": status,
+                    "spec": spec.lower() if spec is not None else "",
+                },
+            )
         )
         methodishStart = re.match(r"([^(]+\()[^)]", anchor["text"][0])
         if methodishStart:
@@ -844,8 +854,8 @@ def transformInclude(lines: list[str], tagName: str, firstLine: str, lineNum: in
                     )
     if path:
         el = "<pre class=include path='{}'".format(h.escapeAttr(path))
-        for i, (k, v) in enumerate(macros.items()):
-            el += " macro-{}='{} {}'".format(i, k, h.escapeAttr(v))
+        for i, (macroName, macroVal) in enumerate(macros.items()):
+            el += " macro-{}='{} {}'".format(i, macroName, h.escapeAttr(macroVal))
         el += f"{lineNumAttr}></pre>"
 
         indent = getWsPrefix(firstLine)
@@ -944,7 +954,7 @@ def transformIncludeRaw(lines: list[str], tagName: str, firstLine: str, lineNum:
     return []
 
 
-def parseInfoTree(lines: list[str], indent: int = 4, lineNum: int | None = 0):
+def parseInfoTree(lines: list[str], indent: int = 4, lineNum: int | None = 0) -> InfoTreeT:
     # Parses sets of info, which can be arranged into trees.
     # Each info is a set of key/value pairs, semicolon-separated:
     # key1: val1; key2: val2; key3: val3
@@ -1047,13 +1057,13 @@ def parseTag(text: str, lineNumber: int | None) -> StartTag | None:
     The text must start with the opening < character.
     """
 
-    def parseerror(index, state):
+    def parseerror(index: int, state: str) -> None:
         m.die(
             f"Tried to parse a start tag from '{text}', but failed at character {index} '{text[index]}' and parse-state '{state}'.",
             lineNum=lineNumber,
         )
 
-    def eof(i, text):
+    def eof(i: int, text: str) -> bool:
         return i >= len(text)
 
     i = 0
