@@ -15,21 +15,23 @@ from functools import partial
 from isodate import Duration, parse_duration
 
 from . import config, constants, datablocks, markdown, h, messages as m, repository, t
-from .DefaultOrderedDict import DefaultOrderedDict
 
 if t.TYPE_CHECKING:
     from .line import Line
 
+    class DurationT:
+        pass
+
 
 class MetadataManager:
     @property
-    def vshortname(self) -> t.Optional[str]:
+    def vshortname(self) -> str | None:
         if self.level:
             return f"{self.shortname}-{self.level}"
         return self.shortname
 
     @property
-    def displayVshortname(self) -> t.Optional[str]:
+    def displayVshortname(self) -> str | None:
         if self.level:
             return f"{self.displayShortname}-{self.level}"
         return self.displayShortname
@@ -38,119 +40,118 @@ class MetadataManager:
         self.hasMetadata: bool = False
 
         # All metadata ever passed to .addData()
-        self.allData: t.DefaultDict[str, t.List[str]] = defaultdict(list)
+        self.allData: t.DefaultDict[str, list[str]] = defaultdict(list)
 
         # required metadata
-        self.abstract: t.List[str] = []
-        self.ED: t.Optional[str] = None
-        self.level: t.Optional[str] = None
-        self.displayShortname: t.Optional[str] = None
-        self.shortname: t.Optional[str] = None
-        self.status: t.Optional[str] = None
-        self.rawStatus: t.Optional[str] = None
+        self.abstract: list[str] = []
+        self.ED: str | None = None
+        self.level: str | None = None
+        self.displayShortname: str | None = None
+        self.shortname: str | None = None
+        self.status: str | None = None
+        self.rawStatus: str | None = None
 
         # optional metadata
         self.advisementClass: str = "advisement"
         self.assertionClass: str = "assertion"
         self.assumeExplicitFor: bool = False
-        self.atRisk: t.List[str] = []
-        self.audience: t.List[str] = []
-        self.blockElements: t.List[str] = []
+        self.atRisk: list[str] = []
+        self.audience: list[str] = []
+        self.blockElements: list[str] = []
         self.boilerplate: config.BoolSet = config.BoolSet(default=True)
-        self.canIUseURLs: t.List[str] = []
-        self.canonicalURL: t.Optional[str] = None
+        self.canIUseURLs: list[str] = []
+        self.canonicalURL: str | None = None
         self.complainAbout: config.BoolSet = config.BoolSet()
-        self.customTextMacros: t.List[t.Tuple[str, str]] = []
-        self.customWarningText: t.List[str] = []
-        self.customWarningTitle: t.Optional[str] = None
+        self.customTextMacros: list[tuple[str, str]] = []
+        self.customWarningText: list[str] = []
+        self.customWarningTitle: str | None = None
         self.date: date = datetime.utcnow().date()
-        self.deadline: t.Optional[date] = None
-        self.defaultHighlight: t.Optional[str] = None
+        self.deadline: date | None = None
+        self.defaultHighlight: str | None = None
         self.defaultBiblioDisplay: str = "index"
-        self.defaultRefStatus: t.Optional[str] = None
-        self.editors: t.List[str] = []
-        self.editorTerm: t.Dict[str, str] = {"singular": "Editor", "plural": "Editors"}
-        self.expires: t.Optional[date] = None
+        self.defaultRefStatus: str | None = None
+        self.editors: list[dict[str, str | None]] = []
+        self.editorTerm: dict[str, str] = {"singular": "Editor", "plural": "Editors"}
+        self.expires: date | None = None
         self.externalInfotrees: config.BoolSet = config.BoolSet(default=False)
-        self.favicon: t.Optional[str] = None
+        self.favicon: str | None = None
         self.forceCrossorigin: bool = False
-        self.group: t.Optional[str] = None
-        self.h1: t.Optional[str] = None
-        self.ignoreCanIUseUrlFailure: t.List[str] = []
-        self.ignoredTerms: t.List[str] = []
-        self.ignoredVars: t.List[str] = []
-        self.implementationReport: t.Optional[str] = None
+        self.group: str | None = None
+        self.h1: str | None = None
+        self.ignoreCanIUseUrlFailure: list[str] = []
+        self.ignoredTerms: list[str] = []
+        self.ignoredVars: list[str] = []
+        self.implementationReport: str | None = None
         self.includeCanIUsePanels: bool = False
         self.includeMdnPanels: bool = False
         self.indent: int = 4
         self.inferCSSDfns: bool = False
-        self.informativeClasses: t.List[str] = []
+        self.informativeClasses: list[str] = []
         self.inlineGithubIssues: bool = False
-        self.inlineTagCommands: t.Dict[str, str] = {}
+        self.inlineTagCommands: dict[str, str] = {}
         self.issueClass: str = "issue"
-        self.issues: t.List[t.Tuple[str, str]] = []
-        self.issueTrackerTemplate: t.Optional[str] = None
+        self.issues: list[tuple[str, str]] = []
+        self.issueTrackerTemplate: str | None = None
         self.lineNumbers: bool = False
         self.linkDefaults: t.LinkDefaultsT = defaultdict(list)
         self.localBoilerplate: config.BoolSet = config.BoolSet(default=False)
-        self.logo: t.Optional[str] = None
-        self.mailingList: t.Optional[str] = None
-        self.mailingListArchives: t.Optional[str] = None
+        self.logo: str | None = None
+        self.mailingList: str | None = None
+        self.mailingListArchives: str | None = None
         self.markupShorthands: config.BoolSet = config.BoolSet(
             ["css", "dfn", "biblio", "markup", "http", "idl", "algorithm"]
         )
-        self.maxToCDepth: t.Union[int, float, None] = float("inf")
+        self.maxToCDepth: int | float | None = float("inf")
         self.metadataInclude: config.BoolSet = config.BoolSet(default=True)
-        self.metadataOrder: t.List[str] = ["*", "!*"]
+        self.metadataOrder: list[str] = ["*", "!*"]
         self.noAbstract: bool = False
         self.noEditor: bool = False
         self.noteClass: str = "note"
-        self.opaqueElements: t.List[str] = ["pre", "xmp", "script", "style"]
+        self.opaqueElements: list[str] = ["pre", "xmp", "script", "style"]
         self.prepTR: bool = False
-        self.previousEditors: t.List[t.Dict[str, t.Optional[str]]] = []
-        self.previousVersions: t.List[str] = []
+        self.previousEditors: list[dict[str, str | None]] = []
+        self.previousVersions: list[dict[str, str]] = []
         self.removeMultipleLinks: bool = False
-        self.repository: t.Optional[repository.Repository] = None
-        self.requiredIDs: t.List[str] = []
+        self.repository: repository.Repository | None = None
+        self.requiredIDs: list[str] = []
         self.slimBuildArtifact: bool = False
-        self.statusText: t.List[str] = []
-        self.testSuite: t.Optional[str] = None
-        self.title: t.Optional[str] = None
+        self.statusText: list[str] = []
+        self.testSuite: str | None = None
+        self.title: str | None = None
         self.toggleDiffs: bool = False
-        self.TR: t.Optional[str] = None
+        self.TR: str | None = None
         self.trackingVectorAltText: str = "(This is a tracking vector.)"
         self.trackingVectorClass: str = "tracking-vector"
-        self.trackingVectorImage: t.Optional[str] = None
+        self.trackingVectorImage: str | None = None
         self.trackingVectorImageHeight: str = "64"
         self.trackingVectorImageWidth: str = "46"
         self.trackingVectorTitle: str = "There is a tracking vector here."
-        self.translateIDs: t.DefaultDict[str, t.List[t.Tuple[str, str]]] = defaultdict(list)
-        self.translations: t.List[t.Dict[str, str]] = []
+        self.translateIDs: dict[str, str] = {}
+        self.translations: list[dict[str, str]] = []
         self.useDfnPanels: bool = True
         self.useIAutolinks: bool = False
-        self.versionHistory: t.List[str] = []
-        self.warning: t.Optional[str] = None
-        self.workStatus: t.Optional[str] = None
+        self.versionHistory: list[str] = []
+        self.warning: str | None = None
+        self.workStatus: str | None = None
         self.wptDisplay: str = "none"
-        self.wptPathPrefix: t.Optional[str] = None
+        self.wptPathPrefix: str | None = None
         self.imgAutoSize: bool = True
 
-        self.otherMetadata = DefaultOrderedDict(list)
+        self.otherMetadata: OrderedDict[str, list[t.NodesT]] = OrderedDict()
 
-        self.manuallySetKeys: t.Set[str] = set()
+        self.manuallySetKeys: set[str] = set()
 
-    def addData(self, key: str, val: str, lineNum: t.Union[None, str, int] = None) -> MetadataManager:
+    def addData(self, key: str, val: str, lineNum: str | int | None = None) -> MetadataManager:
         key = key.strip()
-        if isinstance(val, str):
-            if key in ["Abstract"]:
-                val = val.strip("\n")
-            else:
-                val = val.strip()
+        if key in ["Abstract"]:
+            val = val.strip("\n")
+        else:
+            val = val.strip()
 
         if key.startswith("!"):
             self.allData[key].append(val)
             key = key[1:]
-            self.otherMetadata[key].append(val)
+            self.otherMetadata.setdefault(key, []).append(val)
             return self
 
         if key not in ("ED", "TR", "URL"):
@@ -381,12 +382,14 @@ class MetadataManager:
             macros[name.lower()] = text
 
 
-class ParseFunc(t.Protocol):
-    def __call__(self, key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Any:
-        ...
+if t.TYPE_CHECKING:
+
+    class ParseFunc(t.Protocol):
+        def __call__(self, key: str, val: str, lineNum: str | int | None) -> t.Any:
+            ...
 
 
-def parseDate(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optional[date]:
+def parseDate(key: str, val: str, lineNum: str | int | None) -> date | None:
     if val == "now":
         return datetime.utcnow().date()
     try:
@@ -396,14 +399,14 @@ def parseDate(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optiona
         return None
 
 
-def parseDateOrDuration(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optional[t.Union[date, timedelta]]:
+def parseDateOrDuration(key: str, val: str, lineNum: str | int | None) -> date | timedelta | None:
     if val == "now":
         return datetime.utcnow().date()
     if val == "never" or boolish(val) is False:
         return None
     try:
         if val.startswith("P"):
-            return parse_duration(val)
+            return t.cast(timedelta, parse_duration(val))
         return datetime.strptime(val, "%Y-%m-%d").date()
     except ValueError:
         m.die(
@@ -413,40 +416,40 @@ def parseDateOrDuration(key: str, val: str, lineNum: t.Union[None, str, int]) ->
         return None
 
 
-def canonicalizeExpiryDate(base: date, expires: t.Union[None, timedelta, Duration, datetime, date]) -> t.Optional[date]:
+def canonicalizeExpiryDate(base: date, expires: timedelta | datetime | date | None) -> date | None:
     if expires is None:
         return None
     if isinstance(expires, timedelta):
         return base + expires
     if isinstance(expires, Duration):
-        return base + expires
+        return t.cast(date, base + expires)
     if isinstance(expires, datetime):
         return expires.date()
     if isinstance(expires, date):
         return expires
-    m.die(f"Unexpected expiry type: canonicalizeExpiryDate({base}, {expires})", base, expires)
+    m.die(f"Unexpected expiry type: canonicalizeExpiryDate({base}, {expires})")
     return None
 
 
-def parseLevel(key: str, val: str, lineNum: t.Union[None, str, int]) -> str:
+def parseLevel(key: str, val: str, lineNum: str | int | None) -> str:
     val = val.lower().strip()
     if val == "none":
         return ""
     return val.strip()
 
 
-def parseInteger(key: str, val: str, lineNum: t.Union[None, str, int]) -> int:
+def parseInteger(key: str, val: str, lineNum: str | int | None) -> int:
     return int(val)
 
 
-def parseBoolean(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optional[bool]:
+def parseBoolean(key: str, val: str, lineNum: str | int | None) -> bool | None:
     b = boolish(val)
     if b is None:
         m.die(f"The {key} field must be true/false, yes/no, y/n, or on/off. Got '{val}' instead.", lineNum=lineNum)
     return b
 
 
-def parseSoftBoolean(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Union[None, bool, t.Literal["maybe"]]:
+def parseSoftBoolean(key: str, val: str, lineNum: str | int | None) -> bool | t.Literal["maybe"] | None:
     b = boolish(val)
     if b is not None:
         return b
@@ -456,7 +459,7 @@ def parseSoftBoolean(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.
     return None
 
 
-def boolish(val) -> t.Optional[bool]:
+def boolish(val: str) -> bool | None:
     if val.lower() in ("true", "yes", "y", "on"):
         return True
     if val.lower() in ("false", "no", "n", "off"):
@@ -464,7 +467,7 @@ def boolish(val) -> t.Optional[bool]:
     return None
 
 
-def parseWarning(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optional[t.Tuple[str, ...]]:
+def parseWarning(key: str, val: str, lineNum: str | int | None) -> tuple[str, ...] | None:
     if val.lower() == "obsolete":
         return ("warning-obsolete",)
     if val.lower() == "not ready":
@@ -497,14 +500,14 @@ def parseWarning(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Opti
     return None
 
 
-def parseEditor(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[t.Dict[str, t.Optional[str]]]:
+def parseEditor(key: str, val: str, lineNum: None | str | int) -> list[dict[str, str | None]]:
     pieces = [h.unescape(piece.strip()) for piece in val.split(",")]
 
-    def looksLinkish(string):
-        return re.match(r"\w+:", string) or looksEmailish(string)
+    def looksLinkish(string: str) -> bool:
+        return bool(re.match(r"\w+:", string)) or looksEmailish(string)
 
-    def looksEmailish(string):
-        return re.match(r".+@.+\..+", string)
+    def looksEmailish(string: str) -> bool:
+        return bool(re.match(r".+@.+\..+", string))
 
     data = {
         "name": pieces[0],
@@ -576,15 +579,15 @@ def parseEditor(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[
     return [data]
 
 
-def parseCommaSeparated(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[str]:
+def parseCommaSeparated(key: str, val: str, lineNum: str | int | None) -> list[str]:
     return [term.strip().lower() for term in val.split(",")]
 
 
-def parseIdList(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[str]:
+def parseIdList(key: str, val: str, lineNum: str | int | None) -> list[str]:
     return [term.strip() for term in val.split(",")]
 
 
-def parseLinkDefaults(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.LinkDefaultsT:
+def parseLinkDefaults(key: str, val: str, lineNum: str | int | None) -> t.LinkDefaultsT:
     defaultSpecs: t.LinkDefaultsT = defaultdict(list)
     for default in val.split(","):
         match = re.match(
@@ -611,7 +614,7 @@ def parseLinkDefaults(key: str, val: str, lineNum: t.Union[None, str, int]) -> t
     return defaultSpecs
 
 
-def parseBoilerplate(key: str, val: str, lineNum: t.Union[None, str, int]) -> config.BoolSet:
+def parseBoilerplate(key: str, val: str, lineNum: str | int | None) -> config.BoolSet:
     boilerplate = config.BoolSet(default=True)
     for command in val.split(","):
         pieces = command.lower().strip().split()
@@ -636,7 +639,7 @@ def parseBoilerplate(key: str, val: str, lineNum: t.Union[None, str, int]) -> co
     return boilerplate
 
 
-def parseBiblioDisplay(key: str, val: str, lineNum: t.Union[None, str, int]) -> str:
+def parseBiblioDisplay(key: str, val: str, lineNum: str | int | None) -> str:
     val = val.strip().lower()
     if val in constants.biblioDisplay:
         return val
@@ -644,7 +647,7 @@ def parseBiblioDisplay(key: str, val: str, lineNum: t.Union[None, str, int]) ->
     return constants.biblioDisplay.index
 
 
-def parseRefStatus(key: str, val: str, lineNum: t.Union[None, str, int]) -> str:
+def parseRefStatus(key: str, val: str, lineNum: str | int | None) -> str:
     val = val.strip().lower()
     if val == "dated":
         # Legacy term that used to be allowed
@@ -655,13 +658,13 @@ def parseRefStatus(key: str, val: str, lineNum: t.Union[None, str, int]) -> str:
     return constants.refStatus.current
 
 
-def parseComplainAbout(key: str, val: str, lineNum: t.Union[None, str, int]) -> config.BoolSet:
+def parseComplainAbout(key: str, val: str, lineNum: str | int | None) -> config.BoolSet:
     validLabels = frozenset(["missing-example-ids", "broken-links", "accidental-2119", "missing-exposed"])
     ret = parseBoolishList(key, val.lower(), default=False, validLabels=validLabels, lineNum=lineNum)
     return ret
 
 
-def parseExternalInfotrees(key: str, val: str, lineNum: t.Union[None, str, int]) -> config.BoolSet:
+def parseExternalInfotrees(key: str, val: str, lineNum: str | int | None) -> config.BoolSet:
     return parseBoolishList(
         key,
         val.lower(),
@@ -675,9 +678,9 @@ def parseBoolishList(
     key: str,
     val: str,
     default: bool,
-    validLabels: t.Optional[t.AbstractSet[str]] = None,
-    extraValues: t.Optional[t.Dict[str, bool]] = None,
-    lineNum: t.Union[None, str, int] = None,
+    validLabels: t.AbstractSet[str] | None = None,
+    extraValues: dict[str, bool] | None = None,
+    lineNum: str | int | None = None,
 ) -> config.BoolSet:
     # Parses anything defined as "label <boolish>, label <boolish>" into a BoolSet
     # Supply a list of valid labels if you want to have them checked,
@@ -709,7 +712,7 @@ def parseBoolishList(
     return boolset
 
 
-def parseLinkedText(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[t.Tuple[str, str]]:
+def parseLinkedText(key: str, val: str, lineNum: str | int | None) -> list[tuple[str, str]]:
     # Parses anything defined as "text url, text url, text url" into a list of 2-tuples.
     entries = []
     vals = [v.strip() for v in val.split(",")]
@@ -719,7 +722,7 @@ def parseLinkedText(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.L
     return entries
 
 
-def parseMarkupShorthands(key: str, val: str, lineNum: t.Union[None, str, int]) -> config.BoolSet:
+def parseMarkupShorthands(key: str, val: str, lineNum: str | int | None) -> config.BoolSet:
     # Format is comma-separated list of shorthand category followed by boolean.
     # Output is a boolset of the shorthand categories.
     # TODO: Just call parseBoolistList instead
@@ -748,7 +751,7 @@ def parseMarkupShorthands(key: str, val: str, lineNum: t.Union[None, str, int])
     return ret
 
 
-def parseInlineGithubIssues(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Union[t.Literal[False], str]:
+def parseInlineGithubIssues(key: str, val: str, lineNum: str | int | None) -> t.Literal[False] | str:
     val = val.lower()
     if val in ["title", "full"]:
         return val
@@ -761,7 +764,7 @@ def parseInlineGithubIssues(key: str, val: str, lineNum: t.Union[None, str, int]
     return False
 
 
-def parseTextMacro(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[t.Tuple[str, str]]:
+def parseTextMacro(key: str, val: str, lineNum: str | int | None) -> list[tuple[str, str]]:
     # Each Text Macro line is just a macro name (must be uppercase)
     # followed by the text it expands to.
     try:
@@ -775,7 +778,7 @@ def parseTextMacro(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Li
     return [(name, text)]
 
 
-def parseWorkStatus(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optional[str]:
+def parseWorkStatus(key: str, val: str, lineNum: str | int | None) -> str | None:
     # The Work Status is one of (completed, stable, testing, refining, revising, exploring, rewriting, abandoned).
     val = val.strip().lower()
     if val not in (
@@ -796,7 +799,7 @@ def parseWorkStatus(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.O
     return val
 
 
-def parseRepository(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Optional[repository.Repository]:
+def parseRepository(key: str, val: str, lineNum: str | int | None) -> repository.Repository | None:
     # Shortname followed by url, or just url.
     # If just url, I'll try to recognize the shortname from it; otherwise it's the url again.
     val = val.strip()
@@ -819,7 +822,7 @@ def parseRepository(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.O
     return None
 
 
-def parseTranslateIDs(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Dict[str, str]:
+def parseTranslateIDs(key: str, val: str, lineNum: str | int | None) -> dict[str, str]:
     translations = {}
     for v in val.split(","):
         pieces = v.strip().split()
@@ -831,7 +834,7 @@ def parseTranslateIDs(key: str, val: str, lineNum: t.Union[None, str, int]) -> t
     return translations
 
 
-def parseTranslation(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[t.Dict[str, t.Optional[str]]]:
+def parseTranslation(key: str, val: str, lineNum: str | int | None) -> list[dict[str, str | None]]:
     # Format is <lang-code> <url> [ [ , name <name-in-spec-lang> ] || [ , native-name <name-in-the-lang> ] ]?
     pieces = val.split(",")
     if not (1 <= len(pieces) <= 3):
@@ -864,7 +867,7 @@ def parseTranslation(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.
     return [{"lang-code": langCode, "url": url, "name": name, "native-name": nativeName}]
 
 
-def parseAudience(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[str]:
+def parseAudience(key: str, val: str, lineNum: str | int | None) -> list[str]:
     # WG21 value
     values = [x.strip().upper() for x in val.strip().split(",")]
     if len(values) == 1 and values[0] == "ALL":
@@ -907,7 +910,7 @@ def parseAudience(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Lis
         return []
 
 
-def parseEditorTerm(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Dict[str, str]:
+def parseEditorTerm(key: str, val: str, lineNum: str | int | None) -> dict[str, str]:
     values = [x.strip() for x in val.strip().split(",")]
     if len(values) == 2:
         return {"singular": values[0], "plural": values[1]}
@@ -918,7 +921,7 @@ def parseEditorTerm(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.D
     return {"singular": "Editor", "plural": "Editors"}
 
 
-def parseMaxToCDepth(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Union[int, float]:
+def parseMaxToCDepth(key: str, val: str, lineNum: str | int | None) -> int | float:
     if val.lower() == "none":
         return float("inf")
     try:
@@ -938,12 +941,12 @@ def parseMaxToCDepth(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.
     return v
 
 
-def parseMetadataOrder(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[str]:
+def parseMetadataOrder(key: str, val: str, lineNum: str | int | None) -> list[str]:
     pieces = [x.strip() for x in val.split(",")]
     return pieces
 
 
-def parseWptDisplay(key: str, val: str, lineNum: t.Union[None, str, int]) -> str:
+def parseWptDisplay(key: str, val: str, lineNum: str | int | None) -> str:
     val = val.lower()
     if val in ("none", "inline", "open", "closed"):
         return val
@@ -954,7 +957,7 @@ def parseWptDisplay(key: str, val: str, lineNum: t.Union[None, str, int]) -> str
     return "none"
 
 
-def parsePreviousVersion(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.List[t.Dict[str, str]]:
+def parsePreviousVersion(key: str, val: str, lineNum: str | int | None) -> list[dict[str, str]]:
     biblioMatch = re.match(r"from biblio(\s+\S+)?", val.lower())
     if biblioMatch:
         if biblioMatch.group(1):
@@ -963,13 +966,13 @@ def parsePreviousVersion(key: str, val: str, lineNum: t.Union[None, str, int]) -
     return [{"type": "url", "value": val}]
 
 
-def parseInlineTagCommand(key: str, val: str, lineNum: t.Union[None, str, int]) -> t.Dict[str, str]:
+def parseInlineTagCommand(key: str, val: str, lineNum: str | int | None) -> dict[str, str]:
     tag, _, command = val.strip().partition(" ")
     command = command.strip()
     return {tag: command}
 
 
-def parse(lines: t.Sequence[Line]) -> t.Tuple[t.List[Line], MetadataManager]:
+def parse(lines: t.Sequence[Line]) -> tuple[list[Line], MetadataManager]:
     # Given HTML document text, in the form of an array of text lines,
     # extracts all <pre class=metadata> lines and parses their contents.
     # Returns the text lines, with the metadata-related lines removed,
@@ -1022,7 +1025,7 @@ def parse(lines: t.Sequence[Line]) -> t.Tuple[t.List[Line], MetadataManager]:
     return newlines, md
 
 
-def fromCommandLine(overrides: t.List[str]) -> MetadataManager:
+def fromCommandLine(overrides: list[str]) -> MetadataManager:
     # Given a list of strings representing command-line arguments,
     # finds the args that correspond to metadata keys
     # and fills a MetadataManager accordingly.
@@ -1066,7 +1069,7 @@ def fromJson(data: str, source: str) -> MetadataManager:
     return md
 
 
-def getSpecRepository(doc: t.SpecT) -> t.Optional[repository.Repository]:
+def getSpecRepository(doc: t.SpecT) -> repository.Repository | None:
     """
     Attempts to find the name of the repository the spec is a part of.
     Currently only searches for GitHub repos.
@@ -1101,16 +1104,16 @@ def parseDoc(doc: t.SpecT) -> None:
 
     for el in h.findAll(".replace-with-note-class", doc):
         h.removeClass(el, "replace-with-note-class")
-        h.addClass(el, doc.md.noteClass)
+        h.addClass(doc, el, doc.md.noteClass)
     for el in h.findAll(".replace-with-issue-class", doc):
         h.removeClass(el, "replace-with-issue-class")
-        h.addClass(el, doc.md.issueClass)
+        h.addClass(doc, el, doc.md.issueClass)
     for el in h.findAll(".replace-with-assertion-class", doc):
         h.removeClass(el, "replace-with-assertion-class")
-        h.addClass(el, doc.md.assertionClass)
+        h.addClass(doc, el, doc.md.assertionClass)
     for el in h.findAll(".replace-with-advisement-class", doc):
         h.removeClass(el, "replace-with-advisement-class")
-        h.addClass(el, doc.md.advisementClass)
+        h.addClass(doc, el, doc.md.advisementClass)
 
     if (
         "feedback-header" in doc.md.boilerplate
@@ -1121,7 +1124,7 @@ def parseDoc(doc: t.SpecT) -> None:
         doc.md.issues.append(("Inline In Spec", "#issues-index"))
 
 
-def join(*sources: t.Optional[MetadataManager]) -> MetadataManager:
+def join(*sources: MetadataManager | None) -> MetadataManager:
     """
     MetadataManager is a monoid
     """
@@ -1134,7 +1137,7 @@ def join(*sources: t.Optional[MetadataManager]) -> MetadataManager:
             mdentry = knownKeys[k]
             md.addParsedData(k, getattr(mdsource, mdentry.attrName))
         for k, v in mdsource.otherMetadata.items():
-            md.otherMetadata[k].extend(v)
+            md.otherMetadata.setdefault(k, []).extend(v)
     return md
 
 
@@ -1146,49 +1149,56 @@ class Metadata:
     parse: ParseFunc
 
 
-def joinValue(a, b):
+if t.TYPE_CHECKING:
+    JoinA = t.TypeVar("JoinA")
+    JoinB = t.TypeVar("JoinB")
+
+
+def joinValue(a: JoinA, b: JoinB) -> JoinB:
     return b
 
 
-def joinList(a, b):
-    return a + b
+def joinList(a: t.Sequence[JoinA], b: t.Sequence[JoinB]) -> t.Sequence[JoinA | JoinB]:
+    return a + b  # type: ignore[operator, no-any-return]
 
 
-def joinDict(a, b):
-    x = {}
+def joinDict(a: t.Mapping[str, JoinA], b: t.Mapping[str, JoinB]) -> dict[str, JoinA | JoinB]:
+    x: dict[str, JoinA | JoinB] = {}
     x.update(a)
     x.update(b)
     return x
 
 
-def joinBoolSet(a, b):
+def joinBoolSet(a: config.BoolSet, b: config.BoolSet) -> config.BoolSet:
     x = copy.deepcopy(a)
     x.update(b)
     return x
 
 
-def joinDdList(a, b):
-    x = defaultdict(list)
-    x.update(a)
-    x.update(b)
+def joinDdList(
+    a: defaultdict[str, list[JoinA]], b: defaultdict[str, list[JoinB]]
+) -> defaultdict[str, list[JoinA | JoinB]]:
+    x: defaultdict[str, list[JoinA | JoinB]] = defaultdict(list)
+    x.update(a)  # type: ignore[arg-type]
+    x.update(b)  # type: ignore[arg-type]
     return x
 
 
-def parseLiteral(key: str, val: str, lineNum: t.Union[None, str, int]):
+def parseLiteral(key: str, val: str, lineNum: str | int | None) -> str:
     return val
 
 
-def parseLiteralOrNone(key: str, val: str, lineNum: t.Union[None, str, int]):
+def parseLiteralOrNone(key: str, val: str, lineNum: str | int | None) -> str | None:
     if val.lower() == "none":
         return None
     return val
 
 
-def parseLiteralCaseless(key: str, val: str, lineNum: t.Union[None, str, int]):
+def parseLiteralCaseless(key: str, val: str, lineNum: str | int | None) -> str:
     return val.lower()
 
 
-def parseLiteralList(key: str, val: str, lineNum: t.Union[None, str, int]):
+def parseLiteralList(key: str, val: str, lineNum: str | int | None) -> list[str]:
     return [val]
 
 
